///////////////////////////////////////////////
// 
//  Tekram-S
//  Made for Ludum Dare 48h game design competition, April 2004
//  (theme 'infection')
//
//  Jari Komppa aka Sol 
//  http://iki.fi/sol
// 
///////////////////////////////////////////////
// License
///////////////////////////////////////////////
// 
//     This software is provided 'as-is', without any express or implied
//     warranty.    In no event will the authors be held liable for any damages
//     arising from the use of this software.
// 
//     Permission is granted to anyone to use this software for any purpose,
//     including commercial applications, and to alter it and redistribute it
//     freely, subject to the following restrictions:
// 
//     1. The origin of this software must not be misrepresented; you must not
//        claim that you wrote the original software. If you use this software
//        in a product, an acknowledgment in the product documentation would be
//        appreciated but is not required.
//     2. Altered source versions must be plainly marked as such, and must not be
//        misrepresented as being the original software.
//     3. This notice may not be removed or altered from any source distribution.
// 
// (eg. same as ZLIB license)
//
///////////////////////////////////////////////

#include "globals.h"
#include "screen.h"
#include "gameoverscreen.h"

void GameOverScreen::render()
{
    // render background
    SDL_BlitSurface(mBackground, NULL, gScreen, NULL);

    // Calculate score based on 35 seconds being no action.. 2 minutes maxes out with no sweat
    float normalizedscore = (1 / 35000.0f) * mScore;
    int fx = (int)floor(20 * pow(normalizedscore, 2));    
    if (fx > 627 - 14) 
        fx = 627 - 14;
    int maxx = ((SDL_GetTicks() - mScreenStartTick) / 4);
    int x = fx;
    if (x > maxx) x = maxx;
    simplefillrect(14, 257, 14 + x, 310, TO16B(0, 0x9f, 0));
    int i;
    for (i = 0; i < x; i++)
    {
        int y = (310 - 257) - (gGraph[(i * gGraphMax) / fx] * (310 - 257)) / (26 * 26);
        if (y > (310 - 257)) y = (310 - 257);
        if (y < 0) y = 0;
        simplefillrect(14 + i, 257 + y, 14 + i + 1, 310, TO16B(0, 0x3f, 0));
    }
    SDL_UpdateRect(gScreen, 0, 0, 640, 480);
    SDL_Delay(10);
}

void GameOverScreen::mouseup(int x, int y)
{
    // ignore mouse up for a second.. (otherwise player will most likely click
    // himself out of the game over screen by accident)
    if (SDL_GetTicks() - mScreenStartTick > 1000)
        gChangeState = STATE_MENU;
}

void GameOverScreen::mousedown(int x, int y)
{
}

void GameOverScreen::mousemove(int x, int y)
{
}


GameOverScreen::GameOverScreen(int type)
{
    mType = type;
    SDL_Surface *temp;
    if (type == 1)
        temp = SDL_LoadBMP("gameover.bmp");
    else
        temp = SDL_LoadBMP("gamewin.bmp");
    mBackground = SDL_ConvertSurface(temp, gScreen->format,SDL_SWSURFACE);
    SDL_FreeSurface(temp);
    mScreenStartTick = SDL_GetTicks();
    mScore = mScreenStartTick - gGameStartTick;
    if (mType == 2)
        mScore = 200000;
}

GameOverScreen::~GameOverScreen()
{
    SDL_FreeSurface(mBackground);
}

