///////////////////////////////////////////////
// 
//  Tekram-S
//  Made for Ludum Dare 48h game design competition, April 2004
//  (theme 'infection')
//
//  Jari Komppa aka Sol 
//  http://iki.fi/sol
// 
///////////////////////////////////////////////
// License
///////////////////////////////////////////////
// 
//     This software is provided 'as-is', without any express or implied
//     warranty.    In no event will the authors be held liable for any damages
//     arising from the use of this software.
// 
//     Permission is granted to anyone to use this software for any purpose,
//     including commercial applications, and to alter it and redistribute it
//     freely, subject to the following restrictions:
// 
//     1. The origin of this software must not be misrepresented; you must not
//        claim that you wrote the original software. If you use this software
//        in a product, an acknowledgment in the product documentation would be
//        appreciated but is not required.
//     2. Altered source versions must be plainly marked as such, and must not be
//        misrepresented as being the original software.
//     3. This notice may not be removed or altered from any source distribution.
// 
// (eg. same as ZLIB license)
//
///////////////////////////////////////////////

#include "globals.h"
#include "popupnote.h"
#include "gene.h"
#include "person.h"
#include "screen.h"
#include "gamescreen.h"

GameScreen::GameScreen(int difficulty)
{
    srand(SDL_GetTicks());

    SDL_Surface *temp = SDL_LoadBMP("gamebg.bmp");
    mBackground = SDL_ConvertSurface(temp, gScreen->format,SDL_SWSURFACE);
    SDL_FreeSurface(temp);   

    mVaccineCount = 0;
    mVaccinating = 0;
    mOutbreakPeriod = OUTBREAK_MS;
    switch (difficulty)
    {
    case 0: // 'childish'
        mSize = 2;
        mToolSize = 90;
        mKitSize = 12;
        mInspectOffset = 3;
        break;
    case 1: // 'easy'
        mSize = 3;
        mToolSize = 60;
        mKitSize = 9;
        mInspectOffset = 0;
        break;
    case 2: // 'normal'
        mSize = 4;
        mToolSize = 46;
        mKitSize = 6;
        mInspectOffset = 3;
        break;
    case 3: // 'hard'
        mSize = 5;
        mToolSize = 36;
        mKitSize = 5;
        mInspectOffset = 2;
        break;
    case 4: // 'sick'
        mSize = 7;
        mToolSize = 26;
        mKitSize = 4;
        mInspectOffset = -1;
        break;
    case 5: // 'insane'
        mSize = 11;
        mToolSize = 16;
        mKitSize = 3;
        mInspectOffset = -6;
        break;
    }
    mToolBoard = new Gene(mSize);
    mPopulation = new Person*[26*26];
    mCurrentPerson = mMouseyPerson = -1;
    int i;
    for (i = 0; i < 26*26; i++)
        mPopulation[i] = new Person(mSize);
    mOldVaccines = new Gene*[10];
    for (i = 0; i < 10; i++)
    {
        mOldVaccines[i] = new Gene(mSize);
        if (i & 1)
        {
            int j;
            for (j = 0; j < mSize * mSize; j++)
                mOldVaccines[i]->mGrid[j] = 1;
        }
    }
    mOldVaccineRecentlyUsed = new int[10];
    memset(mOldVaccineRecentlyUsed, 0, sizeof(int) * 10);

    gGameStartTick = SDL_GetTicks();
    gGameLastTick = 0;
    mPopuproot = NULL;
    gGraphMax = 0;
}

void GameScreen::addpopup(int x, int y, int type)
{    
    // we need to add the new popupnote as last in the list so
    // that it gets rendered last, ie. on top of others.
    Popupnote * p = new Popupnote(x,y,type);
    if (mPopuproot == NULL)
    {
        mPopuproot = p;
    }
    else
    {
        Popupnote *pwalker = mPopuproot;
        while (pwalker->mNext != NULL)
            pwalker = pwalker->mNext;
        pwalker->mNext = p;
    }
}


GameScreen::~GameScreen()
{
    delete mToolBoard;
    mToolBoard = NULL;
    int i;
    for (i = 0; i < 10; i++)
        delete mOldVaccines[i];
    delete[] mOldVaccines;
    mOldVaccines = NULL;
    delete[] mOldVaccineRecentlyUsed;
    mOldVaccineRecentlyUsed = NULL;
    for (i = 0; i < 26*26; i++)
    {
        delete mPopulation[i];
        mPopulation[i] = NULL;
    }
    delete[] mPopulation;    
    mPopulation = NULL;
    SDL_FreeSurface(mBackground);   
    Popupnote * pwalker = mPopuproot;
    while (pwalker != NULL)
    {
        Popupnote * last = pwalker;
        pwalker = pwalker->mNext;
        delete last;
    }
}

void GameScreen::disease_tick()
{
    int tick = SDL_GetTicks() - gGameStartTick;
    while (gGameLastTick < tick)
    {
        int i;
        for (i = 0; i < 26*26; i++)
        {
            Person *current = mPopulation[i];
            if (current->diseased > 0)
            {
                // if currently vaccinating this disease, don't let it spread
                if (!(mVaccinating && current->mDisease->compare(mToolBoard)))
                {
                    current->diseased++;
                    if ((rand() % 200) < current->diseased)
                    {
                        int xpos = i % 26;
                        int ypos = i / 26;
                        current->diseased = 1;
                        int direction = rand() & 3;
                        switch (direction)
                        {
                        case 0:
                            xpos--;
                            if (xpos < 0) xpos = 25;
                            break;
                        case 1:
                            ypos--;
                            if (ypos < 0) ypos = 25;
                            break;
                        case 2:
                            xpos++;
                            if (xpos > 25) xpos = 0;
                            break;
                        case 3:
                            ypos++;
                            if (ypos > 25) ypos = 0;
                            break;
                        }
                        Person * target = mPopulation[ypos * 26 + xpos];
                        // if target is being vaccinated, don't disease it
                        if (!(mVaccinating && target->mDisease->compare(mToolBoard)) &&
                            (ypos * 26 + xpos) != mCurrentPerson)
                        {
                            int mutatefactor = MUTATEFACTOR_NEW;
                            if (target->diseased > 0)
                            {
                                mutatefactor = MUTATEFACTOR_SAME;
                                if (!target->mDisease->compare(current->mDisease))
                                {
                                    mutatefactor = MUTATEFACTOR_DIFFERENT;
                                }
                            }
                            else
                            {
                                target->diseased++;
                            }
                            if (target->diseased > 0)
                            {
                                current->mDisease->clone(target->mDisease);
                                if (rand() % 1000 < mutatefactor)
                                {
                                    int i;
                                    for (i = 0; i < mSize * mSize / 3; i++)
                                        target->mDisease->mGrid[rand() % (mSize * mSize)] = rand() & 1;
                                    target->mDisease->generate_color();
                                    int person = ypos * 26 + xpos;
                                    addpopup((person / 26) * 16 + 6 + 8, (person % 26) * 16 + 6 + 52 + 8, 1);
                                }
                            }
                        }
                    }
                }
            }
        }
        int lastoutbreakcount = gGameLastTick / mOutbreakPeriod;
        int newoutbreakcount = (gGameLastTick + DISEASETICK_MS) / mOutbreakPeriod;
        if (lastoutbreakcount != newoutbreakcount || gGameLastTick == 0)
            outbreak();       

        int lastseccount = gGameLastTick / 1000;
        int newseccount = (gGameLastTick + DISEASETICK_MS) / 1000;
        if (lastseccount != newseccount && lastseccount < 600)
        {
            int diseasedcount = 0;
            int i;
            for (i = 0; i < 26 * 26; i++)
                if (mPopulation[i]->diseased > 0)
                    diseasedcount++;
            gGraph[lastseccount] = diseasedcount;
            gGraphMax = lastseccount + 1;
        }

        // tick popups
        Popupnote *pwalker = mPopuproot;
        Popupnote *last = NULL; 
        while (pwalker)
        {
            pwalker->mAge++;
            if (pwalker->mAge == MAX_POPUPNOTE_TICKS)
            {
                if (last == NULL)
                {
                    mPopuproot = pwalker->mNext;
                    delete pwalker;
                    pwalker = mPopuproot;
                }
                else
                {
                    last->mNext = pwalker->mNext;
                    delete pwalker;
                    pwalker = last->mNext;
                }
            }
            else
            {
                last = pwalker;
                pwalker = pwalker->mNext;
            }
        }
        
        gGameLastTick += DISEASETICK_MS;
    }
}

void GameScreen::outbreak()
{
    if (mOutbreakPeriod <= MINOUTBREAK_MS)
    {
        int i;
        for (i = 0; i < 26 * 26; i++)
            if (mPopulation[i]->diseased > 0)
                return;
        gChangeState = STATE_GAMEWIN;
        return;
    }
    // Start from an old, vaccinated disease and mutate it a bit. 
    // (Makes a little more sense to use the old vaccines as startup point)
    int person = rand() % (26*26);
    mPopulation[person]->diseased = 1;
    mOldVaccines[rand() % 10]->clone(mPopulation[person]->mDisease);
    // Still, to make things a bit more interesting, there must be at least one
    // difference from any currently available vaccine.
    int vaccinated = 1;
    while (vaccinated)
    {
        int j;
        for (j = 0; j < mSize; j++)
        {
            int p = rand() % (mSize * mSize);
            mPopulation[person]->mDisease->mGrid[p] = rand() & 1;
        }
        mPopulation[person]->mDisease->generate_color();
        vaccinated = 0;
        for (j = 0; j < 10; j++)
            if (mPopulation[person]->mDisease->compare(mOldVaccines[j]))
                vaccinated = 1;
    }
    mPopulation[person]->mDisease->generate_color();
    mOutbreakPeriod = (int)(mOutbreakPeriod * 0.95f);
    addpopup((person / 26) * 16 + 6 + 8, (person % 26) * 16 + 6 + 52 + 8, 0);
}


void GameScreen::vaccinate()
{
    mVaccineCount++;
    int i;
    for (i = 0; i < 26*26; i++)
    {
        if (mPopulation[i]->diseased > 0 &&
            mPopulation[i]->mDisease->compare(mToolBoard))
        {
            mPopulation[i]->diseased = -1;
        }
    }
    int min = mOldVaccineRecentlyUsed[0];
    int minitem = 0;
    for (i = 0; i < 10; i++)
    {
        if (mOldVaccineRecentlyUsed[i] < min)
        {
            minitem = i;
            min = mOldVaccineRecentlyUsed[i];
        }
        if (mOldVaccines[i]->compare(mToolBoard))
        {
            mOldVaccineRecentlyUsed[i] = mVaccineCount;
            return;
        }
    }
    mToolBoard->clone(mOldVaccines[minitem]);
    mOldVaccineRecentlyUsed[minitem] = mVaccineCount;

}

void GameScreen::render()
{        
    disease_tick();
    if (mVaccinating && 
        mVaccinatingDoneTick < (int)SDL_GetTicks())
    {
        // perform the vaccination
        vaccinate();
        mVaccinating = 0;
    }
    // render background
    SDL_BlitSurface(mBackground, NULL, gScreen, NULL);
       
    // render population
    int diseased_count = 0;
    int i, j, k, l;
    int comparison = (mMouseyPerson != -1) ? mMouseyPerson : mCurrentPerson;
    for (i = 0; i < 26; i++)
    {
        for (j = 0; j < 26; j++)
        {
            int x = 6 + j * 16;
            int y = 6 + 52 + i * 16;
            int idx = j * 26 + i;
            if (comparison != -1 && mPopulation[idx]->diseased > 0 && mPopulation[idx]->mDisease->compare(mPopulation[comparison]->mDisease))
            {
                simplefillrect(x - 1, y - 1, x + 16, y + 16, mPopulation[idx]->mDisease->mGeneColor << 1);
                diseased_count++;
            }
            else
            {
                if (mPopulation[idx]->diseased > 0)
                    diseased_count++;
                if (mPopulation[idx]->diseased < 0)
                    simplefillrect(x, y, x + 14, y + 14, TO16B(0x9f,0x9f,0xbf));
                else
                if (mPopulation[idx]->diseased == 0)
                    simplefillrect(x, y, x + 14, y + 14, TO16B(0x9f,0x9f,0x9f));
                else
                    simplefillrect(x, y, x + 14, y + 14, mPopulation[idx]->mDisease->mGeneColor);

            }
        }
    }

    // render life-o-meter
    if (diseased_count > (26*26 * 0.95f))
    {
        gChangeState = STATE_GAMEOVER;
    }
    else
    {
        float life = diseased_count / (float)(26*26 * 0.95f);    
        int diseasecolor = TO16B(0xff, 0, 0);
        if (diseased_count >= 495 && (SDL_GetTicks() >> 8) & 1)
            diseasecolor = TO16B(0x9f, 0, 0);
        simplefillrect(267, 16, (int)floor(267 + (511 - 267) * life), 36, diseasecolor);
        simplefillrect((int)floor(267 + (511 - 267) * life), 16, 511, 36, TO16B(0, 0xff, 0));
    }

    // render tool board    
    if (mVaccinating)
    {
        int left = ((mVaccinatingDoneTick - SDL_GetTicks()) * mSize * mSize) / (VACCINATION_TIME_MS);
        if (left < 0) left = 0;
        for (i = 0; i < mSize; i++)
        {
            for (j = 0; j < mSize; j++)
            {
                int x = 14 + 428 + j * mToolSize;
                int y = 6 + 268 + i * mToolSize;
                if ((i * mSize + j) - left >= 0)
                {
                    simplefillrect(x, y, x + mToolSize - 3, y + mToolSize - 3, mToolBoard->mGrid[j * mSize + i] ? TO16B(0x9f,0x9f,0xff) : TO16B(0x9f,0xff,0x9f));
                }
                else
                {
                    simplefillrect(x, y, x + mToolSize - 3, y + mToolSize - 3, mToolBoard->mGrid[j * mSize + i] ? TO16B(0x1f,0x1f,0x7f) : TO16B(0x1f,0x7f,0x1f));
                }
            }
        }
    }
    else
    {
        for (i = 0; i < mSize; i++)
        {
            for (j = 0; j < mSize; j++)
            {
                int x = 14 + 428 + j * mToolSize;
                int y = 6 + 268 + i * mToolSize;
                simplefillrect(x, y, x + mToolSize - 3, y + mToolSize - 3, mToolBoard->mGrid[j * mSize + i] ? TO16B(0x3f,0x3f,0x9f) : TO16B(0x3f,0x9f,0x3f));
            }
        }
    }

    // render recent vaccines    
    for (l = 0; l < 2; l++)
    {
        for (k = 0; k < 5; k++)
        {
            for (i = 0; i < mSize; i++)
            {
                for (j = 0; j < mSize; j++)
                {
                    int x = 20 + 535 + j * mKitSize + l * (mKitSize*mSize+mKitSize);
                    int y = 32 + 52 + i * mKitSize + k * (mKitSize*mSize+mKitSize);
                    simplefillrect(x, y, x + mKitSize - 1, y + mKitSize - 1, mOldVaccines[k * 2 + l]->mGrid[j * mSize + i] ? TO16B(0x3f,0x3f,0x9f) : TO16B(0x3f,0x9f,0x3f));
                }
            }
        }
    }

    // render current diseases 
    if (mCurrentPerson != -1 && mPopulation[mCurrentPerson]->diseased > 0)
    {
        for (i = 0; i < mSize; i++)
        {
            for (j = 0; j < mSize; j++)
            {
                int x = 24 + 428 + j * mKitSize*2 + mInspectOffset;
                int y = 32 + 52 + i * mKitSize*2 + 16 + mInspectOffset;
                simplefillrect(x, y, x + mKitSize*2 - 1, y + mKitSize*2 - 1, mPopulation[mCurrentPerson]->mDisease->mGrid[j * mSize + i] ? 
                    //TO16B(0x9f,0x9f,0x1f) : TO16B(0x7f,0x00,0x7f));
                    TO16B(0x7f,0x3f,0x9f) : TO16B(0x7f,0x9f,0x3f));
            }
        }
    }

    if (mMouseyPerson != -1 && mPopulation[mMouseyPerson]->diseased > 0)
    {
        for (i = 0; i < mSize; i++)
        {
            for (j = 0; j < mSize; j++)
            {
                int x = 24 + 428 + j * mKitSize * 2 + mInspectOffset;
                int y = 32 + 52 + i * mKitSize * 2 + 96 + mInspectOffset;
                simplefillrect(x, y, x + mKitSize*2 - 1, y + mKitSize*2 - 1, mPopulation[mMouseyPerson]->mDisease->mGrid[j * mSize + i] ? 
                    //TO16B(0x9f,0x9f,0x1f) : TO16B(0x7f,0x00,0x7f));
                    TO16B(0x7f,0x3f,0x9f) : TO16B(0x7f,0x9f,0x3f));
            }
        }
    }

    // render popups
    Popupnote *pwalker = mPopuproot;
    while (pwalker)
    {
        pwalker->render();
        pwalker = pwalker->mNext;
    }
    
    
    SDL_UpdateRect(gScreen, 0, 0, 640, 480);
    SDL_Delay(10);
}

void GameScreen::do_clear()
{
    if (mVaccinating)
        return;
    // clear to green
    int i;
    for (i = 0; i < mSize * mSize; i++)
        mToolBoard->mGrid[i] = 0;

}

void GameScreen::do_invert()
{
    if (mVaccinating)
        return;
    // swap b/g
    int i;
    for (i = 0; i < mSize * mSize; i++)
        mToolBoard->mGrid[i] = !mToolBoard->mGrid[i];
}

void GameScreen::do_hflip()
{
    if (mVaccinating)
        return;
    // swap horizontal
    Gene * temp = new Gene(mSize);
    mToolBoard->clone(temp);
    int i, j;
    for (i = 0; i < mSize; i++)
        for (j = 0; j < mSize; j++)
            mToolBoard->mGrid[j * mSize + i] = temp->mGrid[((mSize - 1) - j) * mSize + i];
    delete temp;
}

void GameScreen::do_vflip()
{
    if (mVaccinating)
        return;
    // swap vertical
    Gene * temp = new Gene(mSize);
    mToolBoard->clone(temp);
    int i, j;
    for (i = 0; i < mSize; i++)
        for (j = 0; j < mSize; j++)
            mToolBoard->mGrid[j * mSize + i] = temp->mGrid[j * mSize + (mSize - 1) - i];
    delete temp;
}

void GameScreen::mouseup(int x, int y)
{    
    int i,j,l,k;
    // check tool board    
    if (!mVaccinating)
    {
        for (i = 0; i < mSize; i++)
        {
            for (j = 0; j < mSize; j++)
            {            
                int xpos = 14 + 428 + j * mToolSize;
                int ypos = 6 + 268 + i * mToolSize;
                if (x >= xpos &&
                    y >= ypos &&
                    x <  xpos + mToolSize &&
                    y <  ypos + mToolSize)
                    mToolBoard->mGrid[j*mSize+i] = !mToolBoard->mGrid[j*mSize+i];
            }
        }
    }

    for (i = 0; i < 26; i++)
    {
        for (j = 0; j < 26; j++)
        {
            int xpos = 6 + j * 16;
            int ypos = 6 + 52 + i * 16;
            if (x >= xpos &&
                y >= ypos &&
                x <  xpos + 16 &&
                y <  ypos + 16)
            {
                if (mPopulation[j * 26 + i]->diseased > 0)
                    mCurrentPerson = j * 26 + i;
                else
                    mCurrentPerson = -1;
            }
        }
    }

    if (!mVaccinating)
    {
        for (l = 0; l < 2; l++)
        {
            for (k = 0; k < 5; k++)
            {
                int xpos = 20 + 535 + l * (mKitSize*mSize+mKitSize);
                int ypos = 32 + 52 + k * (mKitSize*mSize+mKitSize);
                if (x >= xpos &&
                    y >= ypos &&
                    x <  xpos + (mKitSize*mSize+mKitSize) &&
                    y <  ypos + (mKitSize*mSize+mKitSize))
                {
                    mOldVaccines[k*2+l]->clone(mToolBoard);
                }
            }
        }
    }


    if (!mVaccinating &&
        x >= 428 &&
        y >= 462 &&
        x <= 640 &&
        y <= 480)
    {
        if (x >= 428 && x < 445)
        {
            do_clear();
        }
        if (x >= 445 && x < 462)
        {
            do_invert();
        }
        if (x >= 462 && x < 479)
        {
            do_vflip();
        }
        if (x >= 479 && x < 496)
        {
            do_hflip();
        }
        if (x >= 496)
        {
            do_vaccinate();
        }
    }    
}

void GameScreen::do_vaccinate()
{
    if (!mVaccinating)
    {
        mVaccinating = 1;
        mVaccinatingDoneTick = SDL_GetTicks() + VACCINATION_TIME_MS;
        mToolBoard->generate_color(); // needed for comparison
    }
}

void GameScreen::mousedown(int x, int y)
{
}

void GameScreen::mousemove(int x, int y)
{
    mMouseyPerson = -1;
    int i, j;
    for (i = 0; i < 26; i++)
    {
        for (j = 0; j < 26; j++)
        {
            int xpos = 6 + j * 16;
            int ypos = 6 + 52 + i * 16;
            if (x >= xpos &&
                y >= ypos &&
                x <= xpos + 16 &&
                y <= ypos + 16 &&
                mPopulation[j * 26 + i]->diseased > 0)
                mMouseyPerson = j * 26 + i;
        }
    }

}
