///////////////////////////////////////////////
// 
//  Tekram-S
//  Made for Ludum Dare 48h game design competition, April 2004
//  (theme 'infection')
//
//  Jari Komppa aka Sol 
//  http://iki.fi/sol
// 
///////////////////////////////////////////////
// License
///////////////////////////////////////////////
// 
//     This software is provided 'as-is', without any express or implied
//     warranty.    In no event will the authors be held liable for any damages
//     arising from the use of this software.
// 
//     Permission is granted to anyone to use this software for any purpose,
//     including commercial applications, and to alter it and redistribute it
//     freely, subject to the following restrictions:
// 
//     1. The origin of this software must not be misrepresented; you must not
//        claim that you wrote the original software. If you use this software
//        in a product, an acknowledgment in the product documentation would be
//        appreciated but is not required.
//     2. Altered source versions must be plainly marked as such, and must not be
//        misrepresented as being the original software.
//     3. This notice may not be removed or altered from any source distribution.
// 
// (eg. same as ZLIB license)
//
///////////////////////////////////////////////

#include "globals.h"
#include "screen.h"
#include "gamescreen.h"
#include "gameoverscreen.h"
#include "introscreen.h"
#include "menuscreen.h"

// Globals

SDL_Surface *gScreen;
SDL_Surface *gBackground;
int gGameStartTick;
int gGameLastTick;
Screen * gCurrentScreen;
int gChangeState;
int gCurrentState;
int gDifficulty;
int gGraph[600]; // 10 minutes should be enough for everybody
int gGraphMax;
SDL_Surface *gMutation;
SDL_Surface *gOutbreak;

void init(int fs)
{
    //if ( SDL_Init(SDL_INIT_VIDEO | SDL_INIT_NOPARACHUTE) < 0 ) 
    if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) 
    {
        fprintf(stderr, "Unable to init SDL: %s\n", SDL_GetError());
        exit(1);
    }

    SDL_WM_SetCaption("Tekram-S 1.1",NULL);
    
    atexit(SDL_Quit);
    if (fs)
        gScreen = SDL_SetVideoMode(640, 480, 16, SDL_FULLSCREEN | SDL_SWSURFACE);
    else
        gScreen = SDL_SetVideoMode(640, 480, 16, SDL_SWSURFACE); 
    //
    if (gScreen == NULL) 
    {
        fprintf(stderr, "Unable to set 640x480 video: %s\n", SDL_GetError());
        exit(1);
    }

    gGameStartTick = SDL_GetTicks();
    gGameLastTick = 0;
    gChangeState = STATE_MENU;
    gCurrentState = STATE_NOTCHANGING;
    gCurrentScreen = NULL;
    gDifficulty = 1;
    gGraphMax = 0;
    SDL_Surface *temp = SDL_LoadBMP("outbreak.bmp");
    gOutbreak = SDL_ConvertSurface(temp, gScreen->format,SDL_SWSURFACE);
    SDL_FreeSurface(temp);   
    temp = SDL_LoadBMP("mutation.bmp");
    gMutation = SDL_ConvertSurface(temp, gScreen->format,SDL_SWSURFACE);
    SDL_FreeSurface(temp);   
}

void simplefillrect(int x1, int y1, int x2, int y2, int color)
{
    SDL_Rect tr;
    tr.x = x1;
    tr.y = y1;
    tr.w = x2 - x1;
    tr.h = y2 - y1;
    SDL_FillRect(gScreen, &tr, color);
}

void simplefillquad(int x1, int y1, int x2, int y2, int color, int thickness)
{
    simplefillrect(x1,y1,x1+thickness,y2+thickness,color);
    simplefillrect(x1,y1,x2+thickness,y1+thickness,color);
    simplefillrect(x2,y1,x2+thickness,y2+thickness,color);
    simplefillrect(x1,y2,x2+thickness,y2+thickness,color);
}


int main(int argc, char *argv[])
{         
    init((argc > 1));
    
    while (1)
    {
        if (gChangeState != STATE_NOTCHANGING)
        {
            delete gCurrentScreen;
            gCurrentScreen = NULL;
            switch (gChangeState)
            {
            case STATE_INTRO:
                gCurrentScreen = new IntroScreen();
                break;
            case STATE_MENU:
                gCurrentScreen = new MenuScreen();
                break;
            case STATE_GAME:
                gCurrentScreen = new GameScreen(gDifficulty);
                break;
            case STATE_GAMEOVER:
                gCurrentScreen = new GameOverScreen(1);
                break;
            case STATE_GAMEWIN:
                gCurrentScreen = new GameOverScreen(2);
                break;
            }
            gCurrentState = gChangeState;
            gChangeState = STATE_NOTCHANGING;
        }
        gCurrentScreen->render();
        SDL_Event event;
        while ( SDL_PollEvent(&event) ) 
        {
            switch (event.type) 
            {
            case SDL_KEYUP:
                if (event.key.keysym.sym == SDLK_ESCAPE)
                {
                    if (gCurrentState != STATE_MENU)
                    {
                        gChangeState = STATE_MENU;
                    }
                    else
                    {
                        return 0;
                    }
                }
                break;
            case SDL_KEYDOWN:
                if (event.key.keysym.sym == SDLK_SPACE)
                {
                    if (gCurrentState == STATE_GAME)
                    {
                        ((GameScreen*)gCurrentScreen)->do_vaccinate();
                    }
                }
                if (event.key.keysym.sym == SDLK_z)
                {
                    if (gCurrentState == STATE_GAME)
                    {
                        ((GameScreen*)gCurrentScreen)->do_clear();
                    }
                }
                if (event.key.keysym.sym == SDLK_x)
                {
                    if (gCurrentState == STATE_GAME)
                    {
                        ((GameScreen*)gCurrentScreen)->do_invert();
                    }
                }
                if (event.key.keysym.sym == SDLK_a)
                {
                    if (gCurrentState == STATE_GAME)
                    {
                        ((GameScreen*)gCurrentScreen)->do_vflip();
                    }
                }
                if (event.key.keysym.sym == SDLK_s)
                {
                    if (gCurrentState == STATE_GAME)
                    {
                        ((GameScreen*)gCurrentScreen)->do_hflip();
                    }
                }
                break;
            case SDL_MOUSEBUTTONDOWN:
                if (event.button.button == SDL_BUTTON_LEFT)
                {
                    gCurrentScreen->mousedown(event.motion.x, event.motion.y);
                }
                break;
            case SDL_MOUSEBUTTONUP:
                if (event.button.button == SDL_BUTTON_LEFT)
                {
                    gCurrentScreen->mouseup(event.motion.x, event.motion.y);
                }
                break;
            case SDL_MOUSEMOTION:
                gCurrentScreen->mousemove(event.motion.x, event.motion.y);
                break;
                
                
            case SDL_QUIT:
                return(0);
            }
        }
    }
    return 0;
}

